﻿using Microsoft.Extensions.Logging;
using ShipyardContracts.BusinessLogicsContracts;
using ShipyardContracts.SearchModels;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ShipyardView
{
    public partial class FormAddShip : Form
    {
        private readonly ILogger _logger;
        private readonly IShipLogic _logicShip;
        private readonly IShopLogic _logicShop;
        public FormAddShip(ILogger<FormAddShip> logger, IShipLogic logicShip, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;  
            _logicShip = logicShip;
            _logicShop = logicShop;
        }

        private void FormAddShip_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка списка кораблей для пополнения");
            try
            {
                var list = _logicShip.ReadList(null);
                if (list != null)
                {
                    comboBoxShip.DisplayMember = "ShipName";
                    comboBoxShip.ValueMember = "Id";
                    comboBoxShip.DataSource = list;
                    comboBoxShip.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка кораблей");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            _logger.LogInformation("Загрузка списка магазинов для пополнения");
            try
            {
                var list = _logicShop.ReadList(null);
                if (list != null)
                {
                    comboBoxShop.DisplayMember = "ShopName";
                    comboBoxShop.ValueMember = "Id";
                    comboBoxShop.DataSource = list;
                    comboBoxShop.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка магазинов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(numericUpDownCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxShip.SelectedValue == null)
            {
                MessageBox.Show("Выберите корабль", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Пополнение магазина");
            try
            {
                var operationResult = _logicShop.AddShip(new ShopSearchModel
                {
                    Id = Convert.ToInt32(comboBoxShop.SelectedValue)
                },
                _logicShip.ReadElement(new ShipSearchModel()
                {
                    Id = Convert.ToInt32(comboBoxShip.SelectedValue)
                })!, Convert.ToInt32(numericUpDownCount.Text));
                if (!operationResult)
                {
                    throw new Exception("Ошибка при пополнении магазина. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
