using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using ShipyardBusinessLogic.BusinessLogics;
using ShipyardContracts.BusinessLogicsContracts;
using ShipyardContracts.StoragesContracts;
using ShipyardFileImplement.Implements;

namespace ShipyardView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();

            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IDetailStorage, DetailStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IShipStorage, ShipStorage>();
            services.AddTransient<IShopStorage, ShopStorage>();
            services.AddTransient<IDetailLogic, DetailLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IShipLogic, ShipLogic>();
            services.AddTransient<IShopLogic, ShopLogic>();
            services.AddTransient<FormMain>();
            services.AddTransient<FormDetail>();
            services.AddTransient<FormDetails>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormShip>();
            services.AddTransient<FormShipDetail>();
            services.AddTransient<FormShips>();
            services.AddTransient<FormShop>();
            services.AddTransient<FormShops>();
            services.AddTransient<FormAddShip>();
            services.AddTransient<FormSellShips>();
        }

    }

}